import sys
import typing
import allure
from qrunner.utils.log import logger
from qrunner.utils.config import conf
from qrunner.core.android.driver import AndroidDriver
from qrunner.core.android.element import AndroidElement
from qrunner.core.ios.driver import IosDriver
from qrunner.core.ios.element import IosElement
from qrunner.core.browser.driver import BrowserDriver
from qrunner.core.browser.element import WebElement
from qrunner.core.api.request import HttpRequest, ResponseResult
import jmespath
from jsonschema import validate
from jsonschema.exceptions import ValidationError


class TestCase(HttpRequest):
    """
    测试用例基类，所有测试用例需要继承该类
    """
    def start_class(self):
        """
        Hook method for setup_class fixture
        :return:
        """
        pass

    def end_class(self):
        """
        Hook method for teardown_class fixture
        :return:
        """
        pass

    @classmethod
    def setup_class(cls):
        # 初始化driver
        logger.info('初始化driver')
        # 从配置文件中获取浏览器相关配置（为了支持并发执行）
        platform = conf.get_name('common', 'platform')
        serial_no = conf.get_name('app', 'serial_no')
        browser_name = conf.get_name('web', 'browser_name')

        cls.driver: typing.Union[AndroidDriver, IosDriver, BrowserDriver] = None
        if platform == 'android':
            if serial_no:
                cls.driver = AndroidDriver(serial_no)
            else:
                logger.info('serial_no为空')
                sys.exit()
        elif platform == 'ios':
            if serial_no:
                cls.driver = IosDriver(serial_no)
            else:
                logger.info('serial_no为空')
                sys.exit()
        elif platform == 'browser':
            cls.driver = BrowserDriver(browser_name)
        elif platform == 'api':
            pass
        else:
            logger.info(f'不支持的平台: {platform}')
            sys.exit()
        cls().start_class()

    @classmethod
    def teardown_class(cls):
        logger.info('teardown_class')
        platform = conf.get_name('common', 'platform')
        logger.info(platform)
        if platform == 'browser':
            cls().driver.quit()
        cls().end_class()

    def start(self):
        """
        Hook method for setup_method fixture
        :return:
        """
        pass

    def end(self):
        """
        Hook method for teardown_method fixture
        :return:
        """
        pass

    def setup_method(self):
        self.platform = conf.get_name('common', 'platform')
        if self.platform in ['android', 'ios']:
            self.driver.force_start_app()
        self.start()

    def teardown_method(self):
        self.end()
        if self.platform in ['android', 'ios']:
            self.driver.stop_app()
        if self.platform == 'api':
            self.assertStatusCode(200)
            self.assertPath('code', 0)

    @staticmethod
    def set_title(text):
        """
        设置allure报表中对应用例的标题
        @param text: 测试用例标题
        @return:
        """
        allure.dynamic.title(text)

    def el(self, **kwargs):
        """
        :param kwargs: 元素定位方式
        :return: 根据平台返回对应的元素
        """
        element: typing.Union[AndroidElement, IosElement, WebElement] = None
        if self.platform == 'android':
            element = AndroidElement(**kwargs)
        elif self.platform == 'ios':
            element = IosElement(**kwargs)
        elif self.platform == 'browser':
            element = WebElement(self.driver, **kwargs)
        else:
            logger.info(f'不支持的平台: {self.platform}，暂时只支持android、ios、browser')
            sys.exit()
        return element

    def open(self, url, cookies: list = None):
        """
        访问链接为url的页码
        :param url: 页面链接
        :param cookies: 登录相关的cookie，如[
            {'name': 'xxx', 'value': 'xxxx'}
        ]
        :return:
        """
        self.driver.open_url(url)
        if cookies:
            self.driver.add_cookies(cookies)
            self.driver.refresh()

    def screenshot(self, file_name: str):
        """
        截图并存为文件
        :param file_name: 如test.png
        :return:
        """
        if not file_name.endswith('.png'):
            file_name = file_name + '.png'
        self.driver.screenshot(file_name)

    # def allure_shot(self, file_name: str):
    #     """
    #     截图并上传至allure
    #     :param file_name: 如首页截图
    #     :return:
    #     """
    #     self.driver.upload_pic(file_name)

    def assertStatusCode(self, status_code):
        """
        Asserts the HTTP status code
        """
        assert ResponseResult.status_code == status_code, \
            f'status_code {ResponseResult} != {status_code}'

    def assertSchema(self, schema):
        """
        Assert JSON Schema
        doc: https://json-schema.org/
        """
        try:
            validate(instance=ResponseResult.response, schema=schema)
        except ValidationError as msg:
            assert "Response data" == "Schema data", msg
        else:
            assert True

    # def assertJSON(self, assert_json):
    #     """
    #     Assert JSON data
    #     """
    #     AssertInfo.data = []
    #     diff_json(ResponseResult.response, assert_json)
    #     if len(AssertInfo.data) == 0:
    #         self.assertTrue(True)
    #     else:
    #         self.assertEqual("Response data", "Assert data", msg=AssertInfo.data)

    def assertPath(self, path, value):
        """
        Assert path data
        doc: https://jmespath.org/
        """
        search_value = jmespath.search(path, ResponseResult.response)
        if search_value is None:
            assert path is None, "{} No match".format(path)
        else:
            assert search_value == value, f'{search_value} != {value}'


class Page:
    """
    测试页面基类，所有页面需要继承该类
    """
    def __init__(self, driver, url=None):
        """
        :param driver: 驱动句柄
        :param url: 页面链接
        :param cookies: 登录态相关cookies: [
            {'name': 'xxx', 'value': 'xxxx'},
            {'name': 'xxx', 'value': 'xxxx'},
        ]
        """
        self.platform = conf.get_name('common', 'platform')
        self.driver = driver
        if url is not None:
            self.url = url
            self.open(self.url)

    def el(self, **kwargs):
        """
        :param args: 暂时无用
        :param kwargs: 元素定位方式
        :return: 根据平台返回对应的元素
        """
        element: typing.Union[AndroidElement, IosElement, WebElement] = None
        if self.platform == 'android':
            element = AndroidElement(**kwargs)
        elif self.platform == 'ios':
            element = IosElement(**kwargs)
        elif self.platform == 'browser':
            element = WebElement(self.driver, **kwargs)
        else:
            logger.info(f'不支持的平台: {self.platform}，暂时只支持android、ios、browser')
            sys.exit()
        return element

    def open(self, url, cookies: str = None):
        """
        访问链接为url的页码
        :param url: 页面链接
        :@param cookies: 登录相关的cookie，如[
            {'name': 'xxx', 'value': 'xxx'}
        ]
        :return:
        """
        self.driver.open_url(url)
        if cookies:
            self.driver.add_cookies(cookies)
            self.driver.refresh()

    def screenshot(self, file_name):
        """
        截图并存为文件
        :param file_name: 如test.png
        :return:
        """
        if not file_name.endswith('.png'):
            file_name = file_name + '.png'
        self.driver.screenshot(file_name)

    # def allure_shot(self, file_name):
    #     """
    #     截图并上传至allure
    #     :param file_name: 如首页截图
    #     :return:
    #     """
    #     self.driver.upload_pic(file_name)


class H5Page:
    """
    测试页面基类，所有页面需要继承该类
    """
    def __init__(self, driver):
        """
        :param driver: 驱动句柄
        :param url: 页面链接
        :param cookies: 登录态相关cookies: [
            {'name': 'xxx', 'value': 'xxxx'},
            {'name': 'xxx', 'value': 'xxxx'},
        ]
        """
        self.driver = driver

    def el(self, **kwargs):
        """
        :param args: 暂时无用
        :param kwargs: 元素定位方式
        :return: 根据平台返回对应的元素
        """
        element: WebElement = WebElement(self.driver, **kwargs)
        return element

    def open(self, url, cookies: str = None):
        """
        访问链接为url的页码
        :param url: 页面链接
        :@param cookies: 登录相关的cookie，如[
            {'name': 'xxx', 'value': 'xxx'}
        ]
        :return:
        """
        self.driver.open_url(url)
        if cookies:
            self.driver.add_cookies(cookies)
            self.driver.refresh()

    def screenshot(self, file_name):
        """
        截图并存为文件
        :param file_name: 如test.png
        :return:
        """
        if not file_name.endswith('.png'):
            file_name = file_name + '.png'
        self.driver.screenshot(file_name)

    def allure_shot(self, file_name):
        """
        截图并上传至allure
        :param file_name: 如首页截图
        :return:
        """
        self.driver.upload_pic(file_name)



# @Time    : 2022/1/26 9:35
# @Author  : kang.yang@qizhidao.com
# @File    : test_po_demo.py
import qrunner


class DemoPage(qrunner.Page):
    """
    demo页面
    """
    def __init__(self, driver):
        super().__init__(driver)
        # 元素定义
        self.el_ad = self.el(label='close white big')
        self.el_my = self.el(label='我的')
        self.el_order = self.el(label='我的订单')

    def go_my(self):
        """
        从首页进入我的页
        @return:
        """
        self.el_ad.click()
        self.el_my.click()

    def assert_my(self):
        """
        断言文案'我的订单'存在，以确定进入我的页
        @return:
        """
        assert self.el_order.exists(timeout=3), f'我的订单未找到'
        self.screenshot('我的页')


class TestDemo(qrunner.TestCase):
    """
    demo用例
    """
    def test_case_01(self):
        """
        测试从首页进入我的页
        @return:
        """
        self.set_title('IOS端po模式用例')
        page = DemoPage(self.driver)
        page.go_my()
        page.assert_my()



# @Time    : 2022/2/11 16:52
# @Author  : kang.yang@qizhidao.com
# @File    : mysql_util.py
import pymysql


class BaseDB(object):
    def __init__(self, host, username, password, database):
        self.host = host
        self.username = username
        self.password = password
        self.database = database
        self.con = pymysql.connect(host=self.host,
                                   user=self.username,
                                   password=self.password,
                                   database=self.database,
                                   charset='utf8',
                                   )
        # cursorclass=pymysql.cursors.DictCursor
        self.cursor = self.con.cursor()

    def insert(self, sql):
        try:
            self.cursor.execute(sql)
            self.con.commit()
        except Exception as e:
            print(f'插入失败: {e}')
            self.con.rollback()
        else:
            print('插入成功')
        finally:
            self.close()

    def delete(self, sql):
        try:
            self.cursor.execute(sql)
            self.con.commit()
        except Exception as e:
            print(f'删除失败: {e}')
            self.con.rollback()
        else:
            print('删除成功')
        finally:
            self.close()

    def update(self, sql):
        try:
            self.cursor.execute(sql)
            self.con.commit()
        except Exception as e:
            print(f'更新失败: {e}')
            self.con.rollback()
        else:
            print('更新成功')
        finally:
            self.close()

    def select(self, sql, one=True):
        try:
            self.cursor.execute(sql)
        except Exception as e:
            print(f'查询失败: {e}')
        else:
            print('查询成功,', end=' ')
            if one:
                items = list(self.cursor.fetchone())
                print(f'共查询出: 1 行数据, 第一行数据如下:')
                print(f'{items}\n')
            else:
                items = list(self.cursor.fetchall())
                print(f'共查询出: {len(items)} 行数据, 第一行数据如下:')
                print(f'{items[0]}\n')
            return items
        finally:
            self.close()

    def close(self):
        self.cursor.close()
        self.con.close()


class ApiDB(BaseDB):
    def __init__(self):
        super().__init__(
            host='172.16.3.247',
            username='root',
            password='Wz@888888',
            database='apitest',
        )
        self.table_api = 'api'

    def get(self, one=False):
        sql = f"select * from {self.table_api}"
        print(sql)
        result = self.select(sql, one=one)
        print(result)
        return result

    def get_by_path(self, path):
        sql = f'select * from {self.table_api} where apiPath={path}'
        print(sql)
        result = self.select(sql, one=False)
        return result

    def set(self, api_info: list):
        project = api_info[0]
        moduleName = api_info[1]
        moduleDesc = api_info[2]
        apiPath = api_info[3]
        apiDesc = api_info[4]
        sql = f"insert into {self.table_api}(project, moduleName, moduleDesc, apiPath, apiDesc) " \
              f"values('{project}', '{moduleName}', '{moduleDesc}', '{apiPath}', '{apiDesc}')"
        print(sql)
        self.insert(sql)


if __name__ == '__main__':
    pass












# @Time    : 2022/2/22 9:35
# @Author  : kang.yang@qizhidao.com
# @File    : request.py
import requests
import json as json_util
from qrunner.utils.config import conf
from qrunner.utils.log import logger



IMG = ["jpg", "jpeg", "gif", "bmp", "webp"]


def request(func):
    def wrapper(*args, **kwargs):
        func_name = func.__name__
        print("\n")
        logger.info('-------------- Request -----------------[🚀]')
        try:
            url = list(args)[1]
        except IndexError:
            url = kwargs.get("url", "")
        base_url = conf.get_name('api', 'base_url')
        if (base_url is not None) and ("http" not in url):
            url = base_url + list(args)[1]

        img_file = False
        file_type = url.split(".")[-1]
        if file_type in IMG:
            img_file = True

        logger.debug("[method]: {m}      [url]: {u} \n".format(m=func_name.upper(), u=url))
        auth = kwargs.get("auth", "")
        headers = kwargs.pop("headers", {})
        conf_headers = conf.get_name('api', 'headers')
        if conf_headers != 'None':
            headers.update(json_util.loads(conf_headers))
        kwargs['headers'] = headers
        cookies = kwargs.get("cookies", "")
        params = kwargs.get("params", "")
        data = kwargs.get("data", "")
        json = kwargs.get("json", "")
        if auth != "":
            logger.debug(f"[auth]:\n {auth} \n")
        if headers != "":
            # logger.debug(type(headers))
            logger.debug(f"[headers]:\n {headers} \n")
        if cookies != "":
            logger.debug(f"[cookies]:\n {cookies} \n")
        if params != "":
            logger.debug(f"[params]:\n {params} \n")
        if data != "":
            logger.debug(f"[data]:\n {data} \n")
        if json != "":
            logger.debug(f"[json]:\n {json} \n")

        # running function
        r = func(*args, **kwargs)

        ResponseResult.status_code = r.status_code
        logger.info("-------------- Response ----------------")
        try:
            resp = r.json()
            logger.debug(f"[type]: json \n")
            logger.debug(f"[response]:\n {resp} \n")
            ResponseResult.response = resp
        except BaseException as msg:
            logger.debug("[warning]: {} \n".format(msg))
            if img_file is True:
                logger.debug("[type]: {}".format(file_type))
                ResponseResult.response = r.content
            else:
                logger.debug("[type]: text \n")
                logger.debug(f"[response]:\n {r.text} \n")
                ResponseResult.response = r.text

    return wrapper


class ResponseResult:
    status_code = 200
    response = None


class HttpRequest(object):

    @request
    def get(self, url, params=None, **kwargs):
        base_url = conf.get_name('api', 'base_url')
        if (base_url is not None) and ("http" not in url):
            url = base_url + url
        return requests.get(url, params=params, **kwargs)

    @request
    def post(self, url, data=None, json=None, **kwargs):
        base_url = conf.get_name('api', 'base_url')
        if (base_url is not None) and ("http" not in url):
            url = base_url + url
        return requests.post(url, data=data, json=json, **kwargs)

    @request
    def put(self, url, data=None, **kwargs):
        base_url = conf.get_name('api', 'base_url')
        if (base_url is not None) and ("http" not in url):
            url = base_url + url
        return requests.put(url, data=data, **kwargs)

    @request
    def delete(self, url, **kwargs):
        base_url = conf.get_name('api', 'base_url')
        if (base_url is not None) and ("http" not in url):
            url = base_url + url
        return requests.delete(url, **kwargs)

    @property
    def response(self):
        """
        Returns the result of the response
        :return: response
        """
        return ResponseResult.response

    @property
    def session(self):
        """
        A Requests session.
        """
        s = requests.Session()
        return s

    @staticmethod
    def request(method=None, url=None, headers=None, files=None, data=None,
                params=None, auth=None, cookies=None, hooks=None, json=None):
        """
        A user-created :class:`Request <Request>` object.
        """
        req = requests.Request(method, url, headers, files, data,
                               params, auth, cookies, hooks, json)
        return req



import os
import time

import allure
import requests
from selenium import webdriver
from qrunner.utils.config import conf
from qrunner.utils.log import logger
from qrunner.utils.webdriver_manager_extend import ChromeDriverManager


class H5Config:
    webview_version = '77.0.3865.40'


# 重启chromedriver的装饰器
def relaunch(func):
    def wrapper(self, *args, **kwargs):
        try:
            return func(self, *args, **kwargs)
        except requests.exceptions.ConnectionError as _:
            logger.warning("h5 driver error, relaunch now.")
            self.d = H5Driver(conf.get_name('app', 'serial_no'))
    return wrapper


class H5Driver(object):
    _instance = {}

    def __new__(cls, serial_no=None):
        if not serial_no:
            serial_no = conf.get_name('app', 'serial_no')
        if serial_no not in cls._instance:
            cls._instance[serial_no] = super().__new__(cls)
        return cls._instance[serial_no]

    def __init__(self, serial_no=None, pkg_name=None):
        if not serial_no:
            serial_no = conf.get_name('app', 'serial_no')
        if not pkg_name:
            pkg_name = conf.get_name('app', 'pkg_name')

        logger.info(f'启动H5Driver for {serial_no}')
        options = webdriver.ChromeOptions()
        # options.add_experimental_option('androidDeviceSerial', serial_no)
        options.add_experimental_option('androidPackage', pkg_name)
        options.add_experimental_option('androidUseRunningApp', True)
        # options.add_experimental_option('androidProcess', pkg_name)
        # version = conf.get_name('app', 'webview')
        self.d = webdriver.Chrome(
            executable_path=ChromeDriverManager(version=H5Config.webview_version).install(),
            options=options
        )
        self.d.set_page_load_timeout(60)

    @staticmethod
    def get_instance(cls, serial_no=None):
        if serial_no not in cls._instance:
            logger.info(f'{serial_no} Create h5 driver singleton')
            return H5Driver(serial_no)
        return H5Driver._instance[serial_no]

    @relaunch
    def open_url(self, url):
        logger.info(f'访问: {url}')
        self.d.get(url)

    @relaunch
    def back(self):
        logger.info('返回上一页')
        self.d.back()

    @relaunch
    def screenshot(self, file_name):
        """
        截图并保存到预定路径并上传到allure报告
        @param file_name: foo.png or fool
        @return:
        """
        # 把文件名处理成test.png的样式
        if '.' in file_name:
            file_name = file_name.split(r'.')[0]
        # 截图并保存到当前目录的images文件夹中
        img_dir = os.path.join(os.getcwd(), 'images')
        if os.path.exists(img_dir) is False:
            os.mkdir(img_dir)
        time_str = time.strftime('%Y%m%d%H%M%S')
        file_path = os.path.join(img_dir,
                                 f'{file_name}-{time_str}.png')
        logger.info(f'截图保存至: {file_path}')
        self.d.save_screenshot(file_path)
        # 截图并上传到allure报告
        logger.info(f'截图上传至allure报告')
        allure.attach.file(file_path,
                           attachment_type=allure.attachment_type.PNG,
                           name=f'{file_name}-{time_str}')

    # @relaunch
    # def upload_pic(self, filename):
    #     # allure.attach.file(self.d.get_screenshot_as_png(),
    #     #                    attachment_type=allure.attachment_type.PNG,
    #     #                    name=f'{filename}')
    #     self.d.save_screenshot('tmp.png')
    #     allure.attach.file('tmp.png', attachment_type=allure.attachment_type.PNG,
    #                        name=f'{filename}-{get_time()}')
    #     os.remove('tmp.png')

    @property
    def page_content(self):
        page_source = self.d.page_source
        logger.info(f'获取页面内容: \n{page_source}')
        return page_source

    @relaunch
    def max_window(self):
        logger.info('窗口设置全屏')
        self.d.maximize_window()

    @relaunch
    def set_window_size(self, width, height):
        logger.info('设置窗口长宽')
        self.d.set_window_size(width, height)

    def get_windows(self):
        logger.info(f'获取当前打开的窗口列表')
        return self.d.window_handles

    @relaunch
    def switch_window(self, old_windows):
        logger.info('切换到最新的window')
        current_windows = self.d.window_handles
        newest_window = [window for window in current_windows if window not in old_windows][0]
        self.d.switch_to.window(newest_window)

    @relaunch
    def window_scroll(self, width, height):
        logger.info(f'设置scroll bar的宽高')
        js = "window.scrollTo({w},{h});".format(w=str(width), h=str(height))
        self.d.execute_script(js)

    @relaunch
    def switch_to_frame(self, frame_id):
        logger.info(f'切换到frame {frame_id}')
        self.d.switch_to.frame(frame_id)

    @relaunch
    def switch_to_frame_out(self):
        logger.info('从frame中切出来')
        self.d.switch_to.default_content()

    @relaunch
    def execute_js(self, script, *args):
        logger.info(f'执行js脚本: \n{script}')
        self.d.execute_script(script, *args)

    @relaunch
    def click(self, element):
        logger.info(f'点击元素: {element}')
        self.d.execute_script('arguments[0].click();', element)

    @relaunch
    def quit(self):
        logger.info('退出浏览器')
        self.d.quit()

    @relaunch
    def close(self):
        logger.info('关闭当前页签')
        self.d.close()

    @relaunch
    def add_cookies(self, cookies: list):
        for cookie in cookies:
            self.d.add_cookie(cookie)

    def get_cookies(self):
        logger.info('获取cookies')
        cookies = self.d.get_cookies()
        logger.info(cookies)
        return cookies

    def get_cookie(self, name):
        logger.info(f'获取cookie: {name}')
        cookie = self.d.get_cookie(name)
        logger.info(cookie)
        return cookie

    @relaunch
    def delete_all_cookies(self):
        logger.info('删除所有cookie')
        self.d.delete_all_cookies()

    @relaunch
    def delete_cookie(self, name):
        logger.info(f'删除cookie: {name}')
        self.d.delete_cookie(name)

    @relaunch
    def refresh(self):
        logger.info(f'刷新当前页')
        self.d.refresh()

    def get_title(self):
        logger.info('获取页面标题')
        title = self.d.title
        logger.info(title)
        return title

    def get_url(self):
        logger.info('获取页面url')
        url = self.d.current_url
        logger.info(url)
        return url

    def get_alert_text(self):
        logger.info('获取alert的文本')
        alert_text = self.d.switch_to.alert.text
        logger.info(alert_text)
        return alert_text

    @relaunch
    def accept_alert(self):
        logger.info('同意确认框')
        self.d.switch_to.alert.accept()

    @relaunch
    def dismiss_alert(self):
        logger.info('取消确认框')
        self.d.switch_to.alert.dismiss()








# @Time    : 2022/1/27 15:22
# @Author  : kang.yang@qizhidao.com
# @File    : test_po_demo.py
import qrunner
from qrunner.case import H5Driver


class NativePage(qrunner.Page):
    """
    原生po类demo
    """
    def __init__(self, driver):
        super().__init__(driver)
        # 元素定义
        self.close_ad_btn = self.el(resourceId='id/bottom_btn')
        self.ent_entry = self.el(text='查企业')

    def go_ent_search(self):
        self.close_ad_btn.click()
        self.ent_entry.click()


class H5Page(qrunner.H5Page):
    """
    H5po类demo
    """
    def __init__(self, driver):
        super().__init__(driver)
        # 元素定义
        self.search_entry = self.el(xpath="//div[@class='searchBox']")
        self.search_input = self.el(id_="serchInput")
        self.search_confirm = self.el(className="searchBtn")

    def search_by_key(self, keyword):
        self.search_entry.click()
        self.search_input.send_keys(keyword)
        self.search_confirm.click()

    def assert_result(self, expect):
        assert self.el(xpath=f"//div[@class='cardTitleBox']//div[contains(text(), {expect})]")\
            .exists(timeout=10)
        self.screenshot('搜索结果页')


class TestDemo(qrunner.TestCase):
    """
    测试用例demo
    """
    def test_case_01(self):
        """
        通过关键词查询企业详情信息
        @return:
        """
        keyword = '华为'
        expect = '华为技术有限公司'
        self.set_title('查询企业信息')

        native_page = NativePage(self.driver)
        native_page.go_ent_search()
        h5_page = H5Page(H5Driver())
        h5_page.search_by_key(keyword)
        h5_page.assert_result(expect)



# coding=utf-8
# @Time    : 2022/2/21 11:21
# @Author  : kang.yang@qizhidao.com
# @File    : mongo_util.py
import pymongo


class BaseDB(object):
    def __init__(self, ip, db_name, table_name):
        self.client = pymongo.MongoClient(f"mongodb://{ip}:27017/")
        self.db = self.client[db_name]
        self.table = self.db[table_name]

    def insert_one(self, data: dict):
        print(f'插入数据: {data}')
        try:
            result = self.table.insert_one(data)
        except Exception as e:
            print(e)
        else:
            print('插入成功')
            print(result.inserted_id)
        finally:
            self.client.close()

    def insert_list(self, data: list):
        print(f'插入数据: {data}')
        try:
            result = self.table.insert_many(data)
        except Exception as e:
            print(e)
        else:
            print('插入成功')
            print(result.inserted_ids)
        finally:
            self.client.close()

    def select_all(self):
        print('查询表中所有数据')
        try:
            result = list(self.table.find())
        except Exception as e:
            print(e)
        else:
            print('查询成功')
            # print(result)
            return result
        finally:
            self.client.close()

    def select(self, query_data: dict):
        print(f'查询满足 {query_data} 条件的数据')
        try:
            result = list(self.table.find(query_data))
        except Exception as e:
            print(e)
        else:
            print('查询成功')
            # print(result)
            return result
        finally:
            self.client.close()

    def delete_all(self):
        print(f'删除所有数据')
        try:
            result = self.table.delete_many({})
        except Exception as e:
            print(e)
        else:
            print(result.deleted_count, "个文档已删除")
        finally:
            self.client.close()


class ApiSchema(BaseDB):
    def __init__(self):
        super().__init__(ip='172.16.3.247',
                         db_name='api_test',
                         table_name='schema')

    def get_schema_by_path(self, path: str):
        query = {'path': path}
        return self.select(query)

    def insert_schema(self, data: dict):
        self.insert_one(data)


if __name__ == '__main__':
    data = {'data': {'code': 'Beneficiary', 'name': '疑似最终受益人', 'job': '', 'children': [{'code': 'Beneficiary', 'enterpriseUrl': '/company/companyDetail?keyNo=540b1ed3fb30a4449b2893232fcb25f9&name=深圳市腾讯计算机系统有限公司', '_enterpriseId': '540b1ed3fb30a4449b2893232fcb25f9', 'name': '马化腾', 'enterpriseLogoUrl': 'https://qxb-logo-url.oss-cn-hangzhou.aliyuncs.com/OriginalUrl/618ffc1f7600438ae58a1c1b3dcdef5b.jpg', 'enterpriseName': '深圳市腾讯计算机系统有限公司', 'job': '', 'stockPercent': '54.2857%', 'type': 1, 'lineText': {'stockPercent': '54.2857%'}, 'headLabel': '疑似最终受益人', 'pathDesc': '控制路径(占比54.2857%)', 'pathChains': [{'pathChainName': '马化腾', 'pathChainValue': '54.2857%', 'pathChainType': 1}, {'pathChainName': '深圳市腾讯计算机系统有限公司', 'pathChainType': 2}], 'beneficiaryShare': '54.2857%', 'stockPercentRation': 54.2857, 'beneficiaryCode': ''}], 'stockPercentRation': 0.0}, 'code': 0, 'status': 0, 'success': True, 'msg': '成功', 'hasUse': 0}
    ApiSchema().insert_schema(data)






# @Time    : 2022/2/22 11:32
# @Author  : kang.yang@qizhidao.com
# @File    : test_demo.py
import qrunner


@qrunner.module('查企业-详情')
class TestIndex(qrunner.TestCase):

    @qrunner.title('分页查询行政许可[size={size}]')
    @qrunner.data('size', [10, 20])
    def test_pageQueryAdminLicense(self, size):
        """分页查询行政许可"""
        path = '/qzd-bff-enterprise/qzd/v4/es/law/app/pageQueryAdminLicense'
        pay_load = {
            'source': 'app',
            'current': 1,
            'pageSize': size,
            'eid': 'f202501d-1254-4832-98dd-5162a669b8f2'
        }
        self.post(path, json=pay_load)

        self.assertStatusCode(200)
        self.assertPath('code', 0)



import inspect
import os
import time
import platform

import allure
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.select import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.remote.webelement import WebElement as WE
from qrunner.utils.log import logger
from qrunner.core.browser.driver import relaunch
from qrunner.utils.exceptions import ElementTypeError, NoSuchElementException
from qrunner.core.browser.driver import BrowserDriver


# 支持的定位方式
LOC_LIST = {
    'id_': By.ID,
    'name': By.NAME,
    'linkText': By.LINK_TEXT,
    'tagName': By.TAG_NAME,
    'partialLinkText': By.PARTIAL_LINK_TEXT,
    'className': By.CLASS_NAME,
    'xpath': By.XPATH,
    'css': By.CSS_SELECTOR
}


class WebElement:
    """
    Web元素定义
    """
    def __init__(self, driver, **kwargs):
        self._index = kwargs.pop('index', 0)

        if not kwargs:
            raise ElementTypeError('请输入定位方式')

        if len(kwargs) > 1:
            raise ElementTypeError('请仅指定一种定位方式')

        self.k, self.v = next(iter(kwargs.items()))
        # print(self.k, self.v)
        if self.k not in LOC_LIST.keys():
            raise ElementTypeError(f'不支持的定位方式: {self.k}，仅支持: {LOC_LIST.keys()}')

        self._kwargs = kwargs
        self._element = None
        self.driver: BrowserDriver = driver
        self.d = self.driver.d

    @relaunch
    def _wait(self, timeout=3):
        try:
            WebDriverWait(self.d, timeout=timeout)\
                .until(EC.visibility_of_element_located((LOC_LIST[self.k], self.v)))
            return True
        except Exception:
            return False

    @relaunch
    def _find_element(self, retry=3, timeout=5):
        while not self._wait(timeout=timeout):
            if retry > 0:
                retry -= 1
                logger.info(f'重试 查找元素 {self._kwargs}')
                time.sleep(2)
            else:
                frame = inspect.currentframe().f_back
                caller = inspect.getframeinfo(frame)
                logger.warning(f'【{caller.function}:{caller.lineno}】Not found element {self._kwargs}')
                return None
        elements = self.d.find_elements(LOC_LIST[self.k], self.v)
        return elements

    @relaunch
    def show_element(self, elem):
        """
                Show the elements of the operation
                :param elem:
                """
        style_red = 'arguments[0].style.border="2px solid #FF0000"'
        style_blue = 'arguments[0].style.border="2px solid #00FF00"'
        style_null = 'arguments[0].style.border=""'
        for _ in range(2):
            self.driver.execute_js(style_red, elem)
            time.sleep(0.1)
            self.driver.execute_js(style_blue, elem)
            time.sleep(0.1)
        self.driver.execute_js(style_blue, elem)
        time.sleep(0.3)
        self.driver.execute_js(style_null, elem)

    def get_elements(self, retry=3, timeout=3):
        elements = self._find_element(retry=retry, timeout=timeout)
        if elements is None:
            self.driver.screenshot('元素定位失败')
            raise NoSuchElementException(f'{self._kwargs}-定位失败')
        return elements

    def get_element(self, retry=3, timeout=3):
        elements = self._find_element(retry=retry, timeout=timeout)
        if elements is None:
            self.driver.screenshot('元素定位失败')
            raise NoSuchElementException(f'{self._kwargs}-定位失败')
        else:
            element: WE = elements[self._index]
            self.show_element(element)
        return element

    def exists(self, timeout=1):
        logger.info(f'判断元素: {self._kwargs} 是否存在')
        element = self._find_element(retry=0, timeout=timeout)
        if element is None:
            self.driver.screenshot(f'元素定位失败')
            return False
        return True

    @relaunch
    def click(self):
        logger.info(f'点击元素: {self._kwargs}')
        self.get_element().click()

    @relaunch
    def slow_click(self):
        logger.info(f'移动到元素{self._kwargs}，然后点击')
        elem = self.get_element()
        ActionChains(self.d).move_to_element(elem).click(elem).perform()

    @relaunch
    def right_click(self):
        logger.info(f'右键元素{self._kwargs}')
        elem = self.get_element()
        ActionChains(self.d).context_click(elem).perform()

    @relaunch
    def move_to_elem(self):
        logger.info(f'鼠标移动到元素{self._kwargs}上')
        elem = self.get_element()
        ActionChains(self.d).move_to_element(elem).perform()

    @relaunch
    def click_and_hold(self):
        logger.info(f'长按元素: {self._kwargs}')
        elem = self.get_element()
        ActionChains(self.d).click_and_hold(elem).perform()

    @relaunch
    def drag_and_drop(self, x, y):
        logger.info(f'拖动元素{self._kwargs}到坐标{x},{y}')
        elem = self.get_element()
        action = ActionChains(self.d)
        action.drag_and_drop_by_offset(elem, x, y).perform()

    @relaunch
    def double_click(self):
        logger.info(f'双击元素: {self._kwargs}')
        elem = self.get_element()
        ActionChains(self.d).double_click(elem).perform()

    @relaunch
    def send_keys(self, text):
        logger.info(f'点击元素: {self._kwargs}，然后输入: {text}')
        self.get_element().send_keys(text)

    @relaunch
    def clear_text(self):
        logger.info(f'清空文本')
        self.get_element().clear()

    @relaunch
    def enter(self):
        logger.info(f'选中元素{self._kwargs}点击enter')
        self.get_element().send_keys(Keys.ENTER)

    @relaunch
    def select_all(self) -> None:
        logger.info(f"选中元素{self._kwargs}, ctrl+a.")
        if platform.system().lower() == "darwin":
            self.get_element().send_keys(Keys.COMMAND, "a")
        else:
            self.get_element().send_keys(Keys.CONTROL, "a")

    @relaunch
    def cut(self) -> None:
        logger.info(f"选中元素{self._kwargs}, ctrl+x.")
        if platform.system().lower() == "darwin":
            self.get_element().send_keys(Keys.COMMAND, "x")
        else:
            self.get_element().send_keys(Keys.CONTROL, "x")

    @relaunch
    def copy(self) -> None:
        logger.info(f"选中元素{self._kwargs}, ctrl+c.")
        if platform.system().lower() == "darwin":
            self.get_element().send_keys(Keys.COMMAND, "c")
        else:
            self.get_element().send_keys(Keys.CONTROL, "c")

    @relaunch
    def paste(self) -> None:
        logger.info(f"选中元素{self._kwargs}, ctrl+v.")
        if platform.system().lower() == "darwin":
            self.get_element().send_keys(Keys.COMMAND, "v")
        else:
            self.get_element().send_keys(Keys.CONTROL, "v")

    @relaunch
    def backspace(self) -> None:
        logger.info(f"选中元素{self._kwargs}, backspace.")
        self.get_element().send_keys(Keys.BACKSPACE)

    @relaunch
    def delete(self) -> None:
        logger.info(f"选中元素{self._kwargs}, delete.")
        self.get_element().send_keys(Keys.DELETE)

    @relaunch
    def tab(self) -> None:
        logger.info(f"选中元素{self._kwargs}, tab.")
        self.get_element().send_keys(Keys.TAB)

    @relaunch
    def space(self) -> None:
        logger.info(f"选中元素{self._kwargs}, space.")
        self.get_element().send_keys(Keys.SPACE)

    def get_attr(self, attr_name):
        logger.info(f'获取属性{attr_name}的值')
        value = self.get_element().get_attribute(attr_name)
        logger.info(value)
        return value

    def get_display(self):
        logger.info(f'获取元素{self._kwargs}的display属性')
        displayed = self.get_element().is_displayed()
        logger.info(displayed)
        return displayed

    def get_text(self):
        logger.info(f'获取元素 {self._kwargs} 文本')
        element = self.get_element()
        text = element.text
        logger.info(text)
        return text

    def get_texts(self):
        logger.info(f'获取元素 {self._kwargs} 文本列表')
        elements = self.get_elements()
        texts = [elem.text for elem in elements]
        logger.info(texts)
        return texts

    @relaunch
    def select_index(self, index):
        logger.info(f'选择第 {index} 个下拉列表')
        element = self.get_element()
        select = Select(element)
        select.select_by_index(index)

    @relaunch
    def select_value(self, value):
        logger.info(f'选择id为 {value} 的下拉列表')
        element = self.get_element()
        select = Select(element)
        select.select_by_value(value)

    @relaunch
    def select_text(self, text):
        logger.info(f'选择下拉列表 {text} 选项')
        element = self.get_element()
        select = Select(element)
        select.select_by_value(text)

    @relaunch
    def submit(self):
        logger.info(f'提交表单: {self._kwargs}')
        elem = self.get_element()
        elem.submit()

    @relaunch
    def screenshot(self, file_name):
        """
                截图并保存到预定路径并上传到allure报告
                @param file_name: foo.png or fool
                @return:
                """
        # 把文件名处理成test.png的样式
        if '.' in file_name:
            file_name = file_name.split(r'.')[0]
        # 截图并保存到当前目录的images文件夹中
        img_dir = os.path.join(os.getcwd(), 'images')
        if os.path.exists(img_dir) is False:
            os.mkdir(img_dir)
        time_str = time.strftime('%Y%m%d%H%M%S')
        file_path = os.path.join(img_dir,
                                 f'{file_name}-{time_str}.png')
        logger.info(f'截图保存至: {file_path}')
        self.get_element().screenshot(file_path)
        # 截图并上传到allure报告
        logger.info(f'截图上传至allure报告')
        allure.attach.file(file_path,
                           attachment_type=allure.attachment_type.PNG,
                           name=f'{file_name}-{time_str}')





# @Time    : 2022/1/25 20:52
# @Author  : kang.yang@qizhidao.com
# @File    : run.py

import qrunner


if __name__ == '__main__':
    qrunner.main(
        platform='browser'
    )

'''
:param platform: 平台，如browser、android、ios
:param serial_no: 设备id，如UJK0220521066836、00008020-00086434116A002E
:param pkg_name: 应用包名，如com.qizhidao.clientapp、com.qizhidao.company
:param browser_name: 浏览器类型，如chrome、其他暂不支持
:param case_path: 用例路径
:param rerun: 失败重试次数
:param concurrent: 是否需要并发执行，只支持platform为browser的情况
'''



# @Time    : 2022/2/22 11:32
# @Author  : kang.yang@qizhidao.com
# @File    : run.py
import time
import requests
import qrunner


def get_fingerprint():
    timestamp = time.time()
    app_host = 'https://app-pre.qizhidao.com'
    url = f'{app_host}/confuse/css/font/{timestamp}.css'
    requests.get(url, verify=False)
    return timestamp


if __name__ == '__main__':
    headers = {
        'accessToken': 'eyJhbGciOiJIUzUxMiJ9.ZXlKNmFYQWlPaUpFUlVZaUxDSmhiR2NpT2lKa2FYSWlMQ0psYm1NaU9pSkJNVEk0UTBKRExVaFRNalUySW4wLi5RR05oQWhaWnlwVTBJSVBkMDVpWHpBLlZIeF8wOEhFWVNVZE1pYU5nYnh3dmdBNW1hMTRfa3EzOHNoVlFyR2dOZTJQT1JROXFVcEN5TkswZDBudFlTOWxtSzVWcXFfUDEtYVhhSHRvaFJKckN2RUFXeVA4b0dXXzN5NjB1TmRyeHUyTGtfTmMwY2RGdy1PQWo5b2M5aVQzRnpUcGlBOGxfRENwZnpUa3ZtRUUwb0QzRVFkaUpmM0VnQzU1UHZtWWI5NzlEN2s2R1Q5M0Itc1NHdTZjNVg5Z1ByeGRSTUktQXpaTFllOHFiVi05WXptbDkxenJ5cXJmb1lDOVMwdUNrVmcuS2FUT2RQTnQ2RXdCek5OWUdjb3RyQQ.398_WGQsyrxsqb97EMeTrVIS3B1KEznZmDiVZcdtTUVMcML3FC4KnrmcQLtGxiOSYnv6CvkrF-u2-vas64jc0A',
        'signature': '72ff197401b1ec859a5618bcb0e90a34.1wqJG3H3A6',
        'User-Agent': 'Mozilla/5.0 (iOS; OS/14.6) QZDVName/3.0.2 Device/iPhone XR AppleWebKit/537.36 (KHTML, like Gecko)  Safari/537.36 X/0.29c0c3b080053f818d737a9e8c18822d',
        'Content-Type': 'application/json;charset=UTF-8',
        'Qzd-Fingerprint': get_fingerprint()
    }
    qrunner.main(
        case_path='test_case.py',
        platform='api',
        base_url='https://app-pre.qizhidao.com',
        headers=headers
    )



import os.path
import sys

run_content = """
import qrunner


if __name__ == '__main__':
    qrunner.main(
        platform='android',
        serial_no='UJK0220521066836',
        pkg_name='com.qizhidao.clientapp'
    )

'''
:param platform: 平台，如browser、android、ios
:param serial_no: 设备id，如UJK0220521066836、00008020-00086434116A002E
:param pkg_name: 应用包名，如com.qizhidao.clientapp、com.qizhidao.company
:param browser_name: 浏览器类型，如chrome、其他暂不支持
:param case_path: 用例路径
:param rerun: 失败重试次数
:param concurrent: 是否需要并发执行，只支持platform为browser的情况
'''
"""
case_android_content = """
import qrunner


class TestMy(qrunner.TestCase):
    def test_go_my(self):
        self.element(resourceId='id/bottom_btn').click()
        self.element(resourceId='id/bottom_view', index=3).click()
        assert self.element(text='我的订单').exists(timeout=3)
        self.driver.screenshot('test.png')
"""

require_content = """qrunner
"""

ignore_content = "\n".join(
    ["allure-results/*", "__pycache__/*", "*.pyc", "report.html", ".idea/*"]
)


def init_scaffold_project(subparsers):
    parser = subparsers.add_parser(
        "create", help="Create a new project with template structure."
    )
    parser.add_argument(
        "project_name", type=str, nargs="?", help="Specify new project name."
    )
    return parser


def create_scaffold(project_name):
    """ create scaffold with specified project name.
    """

    def create_folder(path):
        os.makedirs(path)
        msg = f"created folder: {path}"
        print(msg)

    def create_file(path, file_content=""):
        with open(path, "w", encoding="utf-8") as f:
            f.write(file_content)
        msg = f"created file: {path}"
        print(msg)

    create_folder(project_name)
    create_file(
        os.path.join(project_name, "run.py"),
        run_content,
    )
    create_file(
        os.path.join(project_name, "../.gitignore"),
        ignore_content,
    )
    create_file(
        os.path.join(project_name, "requirements.txt"),
        require_content,
    )
    create_file(
        os.path.join(project_name, "test_demo.py"),
        case_android_content,
    )
    # show_tree(project_name)
    return 0


def main_scaffold_project(args):
    sys.exit(create_scaffold(args.project_name))




# @Time    : 2022/1/27 13:47
# @Author  : kang.yang@qizhidao.com
# @File    : test_demo.py
import qrunner
from qrunner.case import H5Driver, wel


class TestDemo(qrunner.TestCase):
    """
    测试用例demo
    """
    def test_case_01(self):
        """
        通过关键词查询企业详情信息
        @return:
        """
        keyword = '华为'
        expect = '华为技术有限公司'

        self.set_title('查询企业信息')
        self.el(resourceId='id/bottom_btn')\
            .click()
        self.el(text='查企业').\
            click()
        driver = H5Driver()
        wel(driver, xpath="//div[@class='searchBox']")\
            .click()
        wel(driver, id_="serchInput").\
            send_keys(keyword)
        wel(driver, className="searchBtn").\
            click()
        assert wel(driver, xpath=f"//div[@class='cardTitleBox']//div[contains(text(), {expect})]")\
            .exists(timeout=5), f'未搜到: {expect}'
        self.screenshot('搜索结果页')



# @Time    : 2022/1/27 10:19
# @Author  : kang.yang@qizhidao.com
# @File    : test_po_demo.py
import qrunner


class DemoPage(qrunner.Page):
    """
    测试po类
    """
    def __init__(self, driver):
        super().__init__(driver)
        # 元素定义
        self.url = 'https://www.qizhidao.com'
        self.login_entry = self.el(xpath="//span[contains(text(), '登录/注册')]")
        self.pwd_login_tab = self.el(xpath="//span[contains(text(), '帐号密码登录')]")
        self.phone_input = self.el(xpath="//input[@placeholder='请输入手机号码']")
        self.pwd_input = self.el(xpath="//input[@placeholder='请输入密码']")
        self.check_box = self.el(xpath="//span[@class='el-checkbox__input']")
        self.login_btn = self.el(xpath="//span[contains(text(), '立即登录')]")
        self.username = self.el(xpath="//span[@class='txt-line']")
        # 访问页面
        self.open(self.url)

    def login_by_pwd(self, phone, password):
        self.login_entry.click()
        self.pwd_login_tab.click()
        self.phone_input.send_keys(phone)
        self.pwd_input.send_keys(password)
        for el in self.check_box.get_elements():
            el.click()
        self.login_btn.click()

    def assert_login_suc(self, expect):
        actual = self.username.get_text()
        assert actual == expect, f'实际结果: {actual} != 预期结果: {expect}'
        self.screenshot('登录成功页')


class TestDemo(qrunner.TestCase):
    """
    测试demo类
    """
    def start(self):
        self.phone = '13652435335'
        self.password = 'wz123456'
        self.expect = '136****5335'

    def test_case_01(self):
        """
        账号密码登录
        @return:
        """
        self.set_title('账号密码登录')
        page = DemoPage(self.driver)
        page.login_by_pwd(self.phone, self.password)
        page.assert_login_suc(self.expect)



from qrunner.running.runner import main
from qrunner.case import TestCase, Page, H5Page
from qrunner.core.h5.driver import H5Driver
from qrunner.decorate import *
from qrunner.utils.log import logger


__version__ = "0.7.5"
__description__ = "全栈自动化测试框架"




# @Time    : 2022/1/25 20:51
# @Author  : kang.yang@qizhidao.com
# @File    : test_demo.py
import qrunner


class TestDemo(qrunner.TestCase):
    """
    测试用例demo类
    """
    def test_case_01(self):
        """
        测试从首页进入我的页
        @return:
        """
        self.set_title('IOS过程式用例')
        self.el(label='close white big')\
            .click()
        self.el(label='我的').click()
        assert self.el(label='我的订单')\
            .exists(timeout=3)
        self.screenshot('我的页')



import json
import os
import sys
import pytest
import webbrowser
from qrunner.utils.log import logger
from qrunner.utils.config import conf


class TestMain(object):
    """
    Support for app and web
    """
    def __init__(self,
                 platform=None,
                 serial_no=None,
                 pkg_name=None,
                 browser_name='chrome',
                 case_path=None,
                 rerun='0',
                 concurrent=False,
                 base_url=None,
                 headers=None,
                 show_report=True
                 ):
        """
        :param platform: 平台，如browser、android、ios、api
        :param serial_no: 设备id，如UJK0220521066836、00008020-00086434116A002E
        :param pkg_name: 应用包名，如com.qizhidao.clientapp、com.qizhidao.company
        :param browser_name: 浏览器类型，如chrome、其他暂不支持
        :param case_path: 用例路径
        :param rerun: 失败重试次数
        :param concurrent: 是否需要并发执行，只支持platform为browser的情况
        :@param base_url: 接口host
        "@param headers: 额外的请求头，{
            "accessToken": "xxxx",
            "signature": "xxxx"
        }
        :@param show_report: 是否在浏览器中打开测试报告
        """

        self.platform = platform
        self.serial_no = serial_no
        self.pkg_name = pkg_name
        self.browser_name = browser_name
        self.case_path = case_path
        if isinstance(rerun, int):
            self.rerun = str(rerun)
        self.rerun = rerun
        self.concurrent = concurrent
        self.base_url = base_url
        self.headers = headers
        self.show_report = show_report

        # 将数据写入全局变量
        if self.platform is not None:
            conf.set_name('common', 'platform', self.platform)
        if self.serial_no is not None:
            conf.set_name('app', 'serial_no', self.serial_no)
        if self.pkg_name is not None:
            conf.set_name('app', 'pkg_name', self.pkg_name)
        if self.browser_name is not None:
            conf.set_name('web', 'browser_name', self.browser_name)
        if self.base_url is not None:
            conf.set_name('api', 'base_url', self.base_url)
        else:
            if self.platform == 'api':
                print('base_url未配置，请配置后重新执行~')
                sys.exit()
        if self.headers is not None:
            conf.set_name('api', 'headers', json.dumps(self.headers))

        # 执行用例
        logger.info('执行用例')
        logger.info(f'平台: {self.platform}')
        cmd_list = [
            '-sv',
            '--reruns', self.rerun,
            '--alluredir', 'allure-results', '--clean-alluredir'
        ]
        if self.case_path:
            cmd_list.insert(0, self.case_path)
        if self.concurrent:
            if self.platform == 'browser' or self.platform == 'api':
                cmd_list.insert(1, '-n')
                cmd_list.insert(2, 'auto')
                cmd_list.insert(3, '--dist=loadscope')
            else:
                logger.info(f'{self.platform}平台不支持并发执行')
                sys.exit()
        logger.info(cmd_list)
        pytest.main(cmd_list)

        # 用例完成后操作
        conf.set_name('api', 'headers', 'None')  # 清除登录态
        os.system('allure generate allure-results -o allure-report --clean')  # 生成报告
        # 自动打开测试报告
        if self.show_report:
            os.system('allure serve allure-results')


main = TestMain


if __name__ == '__main__':
    main()




# @Time    : 2022/1/27 8:59
# @Author  : kang.yang@qizhidao.com
# @File    : test_demo.py
import qrunner


class TestDemo(qrunner.TestCase):
    """
    测试demo类
    """
    def start(self):
        self.phone = '13652435335'
        self.password = 'wz123456'
        self.index_url = 'https://www.qizhidao.com'

    def test_case_01(self):
        """
        账号密码登录
        @return:
        """
        self.set_title('账号密码登录')
        self.open(self.index_url)
        self.el(xpath="//span[contains(text(), '登录/注册')]")\
            .click()
        self.el(xpath="//span[contains(text(), '帐号密码登录')]") \
            .click()
        self.el(xpath="//input[@placeholder='请输入手机号码']") \
            .send_keys(self.phone)
        self.el(xpath="//input[@placeholder='请输入密码']") \
            .send_keys(self.password)
        elem_list = self.el(xpath="//span[@class='el-checkbox__input']")\
            .get_elements()
        for elem in elem_list:
            elem.click()
        self.el(xpath="//span[contains(text(), '立即登录')]") \
            .click()
        expect = '136****5335'
        actual = self.el(xpath="//span[@class='txt-line']").get_text()
        assert actual == expect, f'实际结果: {actual} != 预期结果: {expect}'
        self.screenshot('登录成功页')



# @Time    : 2022/2/22 10:37
# @Author  : kang.yang@qizhidao.com
# @File    : decorate.py
import allure
import pytest


def title(text):
    return allure.title(text)


def module(text):
    return allure.feature(text)


def data(*args, **kwargs):
    return pytest.mark.parametrize(*args, **kwargs)



from flask import Flask, jsonify, render_template, request
from qrunner.utils.mysql_util import ApiDB

app = Flask(__name__)
app.config["JSON_AS_ASCII"] = False


@app.route('/api', methods=['GET', 'POST'])
def api():
    data = request.args
    path = data.get('path')
    data = ApiDB().get_by_path(path)
    return jsonify(data)


if __name__ == '__main__':
    app.run(debug=True)



import inspect
from typing import Union
from wda import Selector
from qrunner.utils.log import logger
from qrunner.utils.config import conf
from qrunner.core.ios.driver import relaunch_wda, IosDriver
from multiprocessing.dummy import Pool as ThreadPool
from selenium.common.exceptions import NoSuchElementException


class IosConfig:
    alerts = ['使用App时允许', '允许', '始终允许']


class IosElement(object):
    """
    IOS原生元素定义
    """
    def __init__(self, *args, **kwargs):
        # 从kwargs中删掉并返回index
        self._index = kwargs.pop('index', 0)
        # 参数初始化
        self._pkg_name = conf.get_name('app', 'pkg_name')
        self._resourceId = kwargs.pop('resourceId', '')
        if self._resourceId:
            kwargs['resourceId'] = f'{self._pkg_name}:{self._resourceId}'
        self._xpath = kwargs.get('xpath', '')
        self._kwargs = kwargs
        self._element: Union[Selector] = None
        self._serial_no = conf.get_name('app', 'serial_no')
        self._driver = IosDriver.get_instance(self._serial_no)
        self._d = self._driver.d

    # 多线程处理异常弹窗
    def handle_alert(self):
        """
        根据不同定位方式进行点击
        @return:
        """
        def click_alert(loc):
            timeout = 2
            try:
                if '//' in loc:
                    self._d.xpath(loc).click(timeout=timeout)
                else:
                    self._d(name=loc).click(timeout=timeout)
            except Exception:
                pass

        # # 弹窗列表设置
        # alert_list = []
        # alert_list.extend(IosConfig.alerts)  # 添加上面定义的默认支持弹窗列表
        # user_set_list = conf.get_name('app', 'alerts').split(',')
        # alert_list.extend(user_set_list)  # 添加用例在running/conf.ini中设置的弹窗列表

        # 多线程执行点击过程
        pool = ThreadPool(10)
        pool.map(click_alert, IosConfig.alerts)

    @relaunch_wda
    def _find_element(self, retry=3, timeout=3):
        """
        循环查找元素，查找失败先处理弹窗后重试
        @param retry: 重试次数
        @param timeout: 单次查找超时时间
        @return:
        """
        self._element = self._d.xpath(self._xpath) if \
            self._xpath else self._d(**self._kwargs)[self._index]
        while not self._element.wait(timeout=timeout):
            if retry > 0:
                retry -= 1
                logger.warning(f'重试 查找元素： {self._kwargs}')
                self.handle_alert()
            else:
                frame = inspect.currentframe().f_back
                caller = inspect.getframeinfo(frame)
                logger.warning(f'【{caller.function}:{caller.lineno}】未找到元素 {self._kwargs}')
                return None
        return self._element

    def get_element(self, retry=3, timeout=3):
        """
        针对元素定位失败的情况，抛出NoSuchElementException异常
        @param retry:
        @param timeout:
        @return:
        """
        element: Union[Selector] = self._find_element(retry=retry, timeout=timeout)
        if element is None:
            self._driver.screenshot(f'元素定位失败')
            raise NoSuchElementException(f'元素: {self._kwargs} 定位失败')
        return element

    @property
    def info(self):
        logger.info(f'获取元素: {self._kwargs} 的所有属性')
        return self.get_element().info

    @property
    def text(self):
        logger.info(f'获取元素: {self._kwargs} 的text属性')
        return self.get_element().text

    @property
    def className(self):
        logger.info(f'获取元素: {self._kwargs} 的className属性')
        return self.get_element().className

    @property
    def name(self):
        logger.info(f'获取元素: {self._kwargs} 的name属性')
        return self.get_element().name

    @property
    def visible(self):
        logger.info(f'获取元素: {self._kwargs} 的visible属性')
        return self.get_element().visible

    @property
    def value(self):
        logger.info(f'获取元素: {self._kwargs} 的value属性')
        return self.get_element().value

    @property
    def label(self):
        logger.info(f'获取元素: {self._kwargs} 的label属性')
        return self.get_element().label

    @property
    def enabled(self):
        logger.info(f'获取元素: {self._kwargs} 的enabled属性')
        return self.get_element().enabled

    @property
    def displayed(self):
        logger.info(f'获取元素: {self._kwargs} 的displayed属性')
        return self.get_element().displayed

    @property
    def bounds(self):
        logger.info(f'获取元素: {self._kwargs} 的bounds属性')
        return self.get_element().bounds

    def exists(self, timeout=1):
        """
        判断元素是否存在当前页面
        @param timeout:
        @return:
        """
        logger.info(f'元素 {self._kwargs},{self._index} 是否存在:')
        element = self._find_element(retry=0, timeout=timeout)
        if element is None:
            self._driver.screenshot(f'元素定位失败')
            return False
        return True

    def wait_gone(self, timeout=10):
        logger.info(f'等待元素{self._kwargs}消失')
        flag = self.get_element().wait_gone(timeout=timeout)
        logger.info(flag)
        return flag

    @relaunch_wda
    def click(self):
        logger.info(f'点击元素: {self._kwargs},{self._index}')
        self.get_element().click()

    @relaunch_wda
    def click_exists(self):
        if self.exists():
            self.click()

    @relaunch_wda
    def clear_text(self):
        logger.info('清除文本: {text}')
        self.get_element().clear_text()

    @relaunch_wda
    def set_text(self, text):
        logger.info(f'输入框 {self._kwargs},{self._index} 输入: {text}')
        self.get_element().set_text(text)

    @relaunch_wda
    def scroll(self, direction=None):
        """
        scroll to make element visiable
        @param: direction，方向，"up", "down", "left", "right"
        @return:
        """
        if direction is not None:
            self.get_element().scroll(direction=direction)
        else:
            self.get_element().scroll()

    @relaunch_wda
    def swipe_left(self):
        self.get_element().swipe("left")

    @relaunch_wda
    def swipe_right(self):
        self.get_element().swipe("right")

    @relaunch_wda
    def swipe_up(self):
        self.get_element().swipe("up")

    @relaunch_wda
    def swipe_down(self):
        self.get_element().swipe("down")

    def child(self, *args, **kwargs):
        return self.get_element().child(*args, **kwargs)






